#include  <math.h>
#include  <iostream>
#include  <fstream>
#include  <list>
#include  "gsl_integrator.H"
using namespace std;



//Compile with: gcc -o DesignI_modified DesignI_modified.C -lm -lgsl -lgslcblas -lstdc++
//Use as: DesignI <alpha> <beta start point> <beta finish point> <beta number of points> <gamma start point> <gamma finish point> <gamma number of points> <Delta> 
//Means that powers of 10 in beta and gamma values are needed

//////////////
struct GOprm
{ 
  float m,n;
  float A, B, G, D;
};


////////////////////////////////////
int func(double t, const double y[],
         double f[], void *params)
{
  GOprm* p = (GOprm*)params; // *params
  
  double xn = pow (y[0], (*p).n);
  double ym = pow (y[1], (*p).m);

  f[0] = (*p).D * ( (*p).B * ( 1.  + (*p).A  * xn)/(1. +  xn + ym) -  y[0]);
  //for 'design1' model
  f[1] = (*p).D * (*p).G * ( 1.  + (*p).A  * xn)/(1. +  xn ) -  y[1];
  //for '10' model
  //f[1] = (*p).D * (*p).G * ( 1.  + (*p).A  * xn)/(1. +  xn + ym) -  y[1];

  return GSL_SUCCESS;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

int main(int argc,char** argv) 
{
  GOprm p;
  p.m = 2;
  p.n = 2;  
  float Bst, Bfin, Bnum, Gst, Gfin, Gnum;
  int bi, gi;
  double final_time = 100.0;
  double dt = 0.1;
  
  p.D = 10.; //hours-1
  p.A = 50;
  Bst = -5.;   //beta start point(log)
  Bfin = 2.;	//beta finish point
  Bnum = 50.;   //beta number of points
  Gst = -3.;	//gamma start point
  Gfin = 2.;	//gamma finish point
  Gnum=50.;	//gamma number of points

  sscanf(argv[1],"%f",&p.A);
  sscanf(argv[2],"%f",&Bst);
  sscanf(argv[3],"%f",&Bfin);
  sscanf(argv[4],"%f",&Bnum);
  sscanf(argv[5],"%f",&Gst);
  sscanf(argv[6],"%f",&Gfin);
  sscanf(argv[7],"%f",&Gnum);
  sscanf(argv[8],"%f",&p.D);  

  float dB=(Bfin-Bst)/Bnum;
  float dG=(Gfin-Gst)/Gnum;

  if ( argc > 9 ) {
    sscanf(argv[9],"%f",&p.n);
    sscanf(argv[10],"%f",&p.m);

  }

  gsl_integrator I(2,func,&p);

  char out_file[60] = "DesingI.txt";
  ofstream fout(out_file);
  
  for (bi=0; bi<Bnum; bi++) {
	for (gi=0; gi<Gnum; gi++) 	{
		p.B = pow (10, Bst+dB*bi);
		p.G = pow (10, Gst+dG*gi);
		  //Integrate:
		  vector<double> y(2,4.);
		  double t = 0;
		  bool increasing0 = false;
		  bool increasing1 = false;
			      
		  bool decreasing0 = false;
		  bool decreasing1 = false;
		  int peaks = 0;
		  double y0_before, y0_after, t0_before, y1_before, y1_after, t1_before ;

		  //Transitory:
		  while ( t < final_time )
		    {
		      		  
		      y1_before =  y[1];
		      I.advance(y,t,dt);
		      y1_after = y[1];
		      
		      if ( y1_after > y1_before) increasing0 = true;
		      if ( y1_after < y1_before ) decreasing0 = true;
		      if ( decreasing0 && increasing0 )
			{
			  increasing0 = false;
			  decreasing0 = false;
			} 
		    }
		  
		  //Check oscillator:	      		  
		  y0_before =  y[0];
		  t = 0.0;
		  int stationarity = 0;
		  peaks = 0;
		  while ( (t < final_time + 10000.)  && stationarity < 100 )
		    {		      
		      I.advance(y,t,dt);

		      y0_after = y[0];
		      if (fabs(y0_after - y0_before) < 1e-5) stationarity += 1;
		      else stationarity = 0;
		      
		      if ( y0_after > y0_before) increasing0 = true;
		      if ( y0_after < y0_before ) decreasing0 = true;
		      if (decreasing0 && increasing0)
			{
			  peaks += 1;
			  increasing0 = false;
			  decreasing0 = false;
			}     
		      y0_before =  y[0];
		    }
		  
		  bool oscillator = false;
		  if ( peaks > 1 && stationarity < 100)
		    oscillator = true;
		  

		  //Period and amplitude characteristics:
		  double Period0, Amplitude0, VarAmpl0, VarPer0;
		  double Period1, Amplitude1, VarAmpl1, VarPer1;
		  if (oscillator) {  
		    list <double> Peaks0, PeakTimes0, Peaks1, PeakTimes1;
		    int nr_peaks0 = 0;
		    int nr_peaks1 = 0;
		    t = 0.0;
		    while ( (t < final_time + 300.) || nr_peaks0 < 50 )
		      {    
			y0_before =  y[0];
			t0_before = t;			
			y1_before =  y[1];
			t1_before = t;
			
			I.advance(y,t,dt);
			y0_after = y[0];
			y1_after = y[1];
			
			if (fabs(y1_after - y1_before) < 1e-5) stationarity += 1;
			else stationarity = 0;
				    
			if (increasing0)
			  {
			    if ( y0_after < y0_before ) 
			      {
				decreasing0 = true;
				Peaks0.push_back( y0_before );
				PeakTimes0.push_back( t0_before );
				increasing0 = false;
				decreasing0 = false;
				nr_peaks0 += 1;
			      }
			  }
			if (increasing1)
			  {
			    if ( y1_after < y1_before ) 
			      {
				Peaks1.push_back( y1_before );
				PeakTimes1.push_back( t1_before );
				nr_peaks1 += 1;
				increasing1 = false;
				decreasing1 = false;
			      }
			  }
			if ( y0_after > y0_before) increasing0 = true;
			if ( y1_after > y1_before) increasing1 = true;
		      }
		    
		    
		    //Get average amplitude and period for X:
		    list <double>::iterator Am0 = Peaks0.begin();
		    list <double>::iterator Per0 = PeakTimes0.begin();
		
		    int i = 0;
		    double sum_Am = 0.0, sum_Per=0.0, sumSQ_Am=0.0, sumSQ_Per = 0.0;
		    double last_TP;
		    for(; Am0 != Peaks0.end(); Am0++)
		      {
			assert ( Per0 != PeakTimes0.end() ); 
			sum_Am += *Am0;
			sumSQ_Am += (*Am0) * (*Am0);
				    
			if (i != 0) { sum_Per += *Per0 - last_TP; 
			  sumSQ_Per += (*Per0 - last_TP)*(*Per0 - last_TP); }
				    
			last_TP = *Per0 ;
			i++;
			Per0++;
		      }
		    assert ( i == nr_peaks0 );
				    
		    Period0 = sum_Per/i ;
		    Amplitude0 = sum_Am/i;		
		    VarAmpl0 = ( (sumSQ_Am - i *  Amplitude0 * Amplitude0)/(i-1) );
		    VarPer0 = ( (sumSQ_Per - i *  Period0 * Period0)/(i-1) );
		    
		   
		   		
		    //Get average amplitude and period for Y:
		    list <double>::iterator Am1 = Peaks1.begin();
		    list <double>::iterator Per1 = PeakTimes1.begin();
		
		    i = 0;
		    sum_Am = 0.0, sum_Per=0.0, sumSQ_Am=0.0, sumSQ_Per = 0.0;
		    for(; Am1 != Peaks1.end(); Am1++)
		      {
			assert ( Per1 != PeakTimes1.end() ); 
			sum_Am += *Am1;
			sumSQ_Am += (*Am1) * (*Am1);
				    
			if (i != 0) { sum_Per += *Per1 - last_TP; 
			  sumSQ_Per += (*Per1 - last_TP)*(*Per1 - last_TP); }
				    
			last_TP = *Per1 ;
			i++;
			Per1++;
		      }
		    assert ( i == nr_peaks1 );
				    
		    Period1 = sum_Per/i ;
		    Amplitude1 = sum_Am/i;		
		    VarAmpl1 = ( (sumSQ_Am - i *  Amplitude1 * Amplitude1)/(i-1) );
		    VarPer1 = ( (sumSQ_Per - i *  Period1 * Period1)/(i-1) );		
		    
		    fout << Period1 << " ";

		  }
		  //
		  else //not oscillator:
		    {	
		      fout << "NaN" << " ";
		      		      
		    }
				  
		  		    
		  
	}
	fout << "\n";
   }
  
}
